@@ -17,18 +17,20 @@
 # satpy.  If not, see <http://www.gnu.org/licenses/>.
 """Interface to CLAVR-X HDF4 products."""
 
-import os
 import logging
-
-import numpy as np
-import netCDF4
-import xarray as xr
+import os
 from glob import glob
-from satpy.readers.hdf4_utils import HDF4FileHandler, SDS
-from satpy.readers.file_handlers import BaseFileHandler
-from satpy import CHUNK_SIZE
-from pyresample import geometry
 from pathlib import Path
+from typing import Optional
+
+import netCDF4
+import numpy as np
+import xarray as xr
+from pyresample import geometry
+
+from satpy import CHUNK_SIZE
+from satpy.readers.file_handlers import BaseFileHandler
+from satpy.readers.hdf4_utils import SDS, HDF4FileHandler
 
 LOG = logging.getLogger(__name__)
 
@@ -82,7 +84,7 @@ def _get_platform(platform: str) -> str:
     return platform
 
 
-def _get_rows_per_scan(sensor: str) -> str:
+def _get_rows_per_scan(sensor: str) -> Optional[int]:
     """Get number of rows per scan."""
     for k, v in ROWS_PER_SCAN.items():
         if sensor.startswith(k):
@@ -90,18 +92,19 @@ def _get_rows_per_scan(sensor: str) -> str:
     return None
 
 
+def _remove_attributes(attrs: dict) -> dict:
+    """Remove attributes that described data before scaling."""
+    old_attrs = ['unscaled_missing', 'SCALED_MIN', 'SCALED_MAX',
+                 'SCALED_MISSING']
+
+    for attr_key in old_attrs:
+        attrs.pop(attr_key, None)
+    return attrs
+
+
 class _CLAVRxHelper:
     """A base class for the CLAVRx File Handlers."""
 
-    def _remove_attributes(attrs: dict) -> dict:
-        """Remove attributes that described data before scaling."""
-        old_attrs = ['unscaled_missing', 'SCALED_MIN', 'SCALED_MAX',
-                     'SCALED_MISSING']
-
-        for attr_key in old_attrs:
-            attrs.pop(attr_key, None)
-        return attrs
-
     @staticmethod
     def _scale_data(data_arr: xr.DataArray, scale_factor: float, add_offset: float) -> xr.DataArray:
         """Scale data, if needed."""
@@ -141,7 +144,7 @@ class _CLAVRxHelper:
                 data = data.where((data >= valid_min) & (data <= valid_max))
             attrs['valid_range'] = [valid_min, valid_max]
 
-        data.attrs = _CLAVRxHelper._remove_attributes(attrs)
+        data.attrs = _remove_attributes(attrs)
 
         return data
 
@@ -179,11 +182,11 @@ class _CLAVRxHelper:
 
     @staticmethod
     def _find_input_nc(filename: str, l1b_base: str) -> str:
-        filename = Path(filename)
-        dirname = filename.parent
-        l1b_filenames = dirname.joinpath(l1b_base + '.nc')
-        if l1b_filenames.exists():
-            return l1b_filenames
+        file_path = Path(filename)
+        dirname = file_path.parent
+        l1b_filename = dirname.joinpath(l1b_base + '.nc')
+        if l1b_filename.exists():
+            return str(l1b_filename)
 
         glob_pat = os.path.join(dirname, l1b_base + '*R20*.nc')
         LOG.debug("searching for {0}".format(glob_pat))
