@@ -75,7 +75,7 @@ class VIIRSActiveFiresFileHandler(NetCDF4FileHandler):
             data.attrs['units'] = 'K'
 
         data.attrs["platform_name"] = PLATFORM_MAP.get(self.filename_info['satellite_name'].upper(), "unknown")
-        data.attrs["sensor"] = "VIIRS"
+        data.attrs["sensor"] = self.sensor_name
 
         return data
 
@@ -92,7 +92,7 @@ class VIIRSActiveFiresFileHandler(NetCDF4FileHandler):
     @property
     def sensor_name(self):
         """Name of sensor for this file."""
-        return self["sensor"]
+        return self["sensor"].lower()
 
     @property
     def platform_name(self):
